# 修复日志
# 2025-04-03 修复了从类似MAX导入到SU的这类模型无法正确识别坐标位置的问题
# 插件信息
PLUGIN_NAME = '另存选定对象'
PLUGIN_VERSION = '1.0.1'
PLUGIN_AUTHOR = '刘涛'
PLUGIN_COPYRIGHT = '© 2025 liutao91.cn'
PLUGIN_DESCRIPTION = '可将选定对象（群组或组件）另存为单独的模型文件、支持多选。'

# 注册插件
extension = SketchupExtension.new(PLUGIN_NAME, File.join(File.dirname(__FILE__), 'T_save_selected_as_model.rb'))
extension.version = PLUGIN_VERSION
extension.creator = PLUGIN_AUTHOR
extension.copyright = PLUGIN_COPYRIGHT
extension.description = PLUGIN_DESCRIPTION

# 注册到扩展程序管理器
Sketchup.register_extension(extension, true)

# 创建工具栏和菜单（仅在插件启用时加载）
unless file_loaded?(__FILE__)
  # 添加工具栏图标
  icon_path = File.join(File.dirname(__FILE__), 'T_save_selected_as_model', 'save_selected.png')
  cmd = UI::Command.new(PLUGIN_NAME) { save_selected_as_model }
  cmd.small_icon = icon_path
  cmd.large_icon = icon_path
  cmd.tooltip = PLUGIN_NAME
  cmd.status_bar_text = PLUGIN_DESCRIPTION

  # 创建工具栏
  toolbar = UI::Toolbar.new(PLUGIN_NAME)
  toolbar.add_item(cmd)
  toolbar.show

  # 创建插件菜单
  plugins_menu = UI.menu('Plugins') # 获取 SketchUp 的 Plugins 菜单
  plugins_menu.add_item(cmd) # 将插件命令添加到菜单中

  # 标记插件已加载
  file_loaded(__FILE__)
end

# 插件主逻辑
def save_selected_as_model 
  # 获取当前活动的模型
  model = Sketchup.active_model

  # 获取用户当前选中的内容
  selection = model.selection

  if selection.empty?
    UI.messagebox("请先选中一个或多个对象（群组或组件）。")
    return
  end

  # 检查选中对象是否是群组或组件，或者包含其他类型（如线、面）
  valid_entities = selection.select { |entity| entity.is_a?(Sketchup::Group) || entity.is_a?(Sketchup::ComponentInstance) }
  invalid_entities = selection.reject { |entity| entity.is_a?(Sketchup::Group) || entity.is_a?(Sketchup::ComponentInstance) }

  # 如果选中对象中有无效的类型（线、面等）
  if valid_entities.empty?
    UI.messagebox("选中的对象必须是群组或组件。")
    return
  elsif !invalid_entities.empty?
    UI.messagebox("选中的对象包含无效类型（如线或面）。请确保只选中群组或组件。")
    return
  end

  # 创建默认插件目录
  plugin_dir = File.join(File.dirname(__FILE__), 'T_save_selected_as_model')
  FileUtils.mkdir_p(plugin_dir)

  # 设置配置文件路径
  settings_file = File.join(plugin_dir, 'settings.txt')

  # 读取保存的参数
  saved_data = if File.exist?(settings_file)
                 file_content = File.read(settings_file).strip
                 file_content.split('|')
               else
                 nil
               end

  saved_data ||= ['保持当前位置', '分别输出']  # 如果读取失败，使用默认值
  saved_position_option, saved_export_type = saved_data

  # 弹出参数设置框
  prompts = ['对象位置：', '导出类型：']
  defaults = [saved_position_option, saved_export_type]
  options = [
    '保持当前位置|对象坐标原点|对象中心上|对象中心|对象中心下',
    '分别输出|整体输出'
  ]
  user_input = UI.inputbox(prompts, defaults, options, '参数设置')

  # 检查用户输入是否有效
  unless user_input.is_a?(Array) && user_input.size == 2
    return
  end

  position_option, export_type = user_input

  # 保存参数到文件
  File.open(settings_file, 'w') do |file|
    file.puts([position_option, export_type].join('|'))
  end

  if export_type == '整体输出'
    # 计算所有选中对象的共同原点和最小坐标
    bounds = Geom::BoundingBox.new
    valid_entities.each { |entity| bounds.add(entity.bounds) }

    if bounds.valid?
      origin_transformation = case position_option
      when '保持当前位置'
        Geom::Transformation.new
      when '对象坐标原点'
        new_origin = Geom::Point3d.new(bounds.min.x, bounds.min.y, bounds.min.z)
        Geom::Transformation.new(new_origin.vector_to(Geom::Point3d.new))
      when '对象中心上'
        center_top = Geom::Point3d.new(bounds.center.x, bounds.center.y, bounds.max.z)
        Geom::Transformation.new(center_top.vector_to(Geom::Point3d.new))
      when '对象中心'
        center = bounds.center
        Geom::Transformation.new(center.vector_to(Geom::Point3d.new))
      when '对象中心下'
        center_bottom = Geom::Point3d.new(bounds.center.x, bounds.center.y, bounds.min.z)
        Geom::Transformation.new(center_bottom.vector_to(Geom::Point3d.new))
      else
        Geom::Transformation.new
      end

      # 保存到指定目录，设置文件类型为 SketchUp 模型 (*.skp)
      output_path = UI.savepanel('保存选中对象为模型', nil, 'SketchUp 模型|*.skp')

      def file_locked?(file_path)
        begin
          # 尝试以写模式打开文件
          file = File.open(file_path, "r+")
          file.close
          false # 如果能够打开文件，说明没有被占用
        rescue => e
          true # 如果抛出异常，说明文件被占用
        end
      end
      
      if output_path
        # 检查文件是否已经被占用
        if File.exist?(output_path) && file_locked?(output_path)
          UI.messagebox("无法保存文件，因为它正在被其他程序占用：#{output_path}")
          return
        end
      
        # 保存文件
        definition = model.definitions.add('Temporary Export')
        valid_entities.each do |entity|
          new_instance = nil
          if entity.is_a?(Sketchup::Group)
            new_instance = definition.entities.add_instance(entity.entities.parent, origin_transformation * entity.transformation)
          elsif entity.is_a?(Sketchup::ComponentInstance)
            new_instance = definition.entities.add_instance(entity.definition, origin_transformation * entity.transformation)
          end
          # 保留外部材质
          new_instance.material = entity.material if new_instance
        end
      
        begin
          status = definition.save_as(output_path)
          if status
            UI.messagebox("模型已保存到：#{output_path}")
          else
            UI.messagebox('保存失败，请检查路径和文件名是否有效。')
          end
        rescue => e
          UI.messagebox("保存失败，出现错误：#{e.message}")
        end
      
        # 删除临时组件定义
        model.definitions.remove(definition)
      end
      
    else
      UI.messagebox('无法计算所有选中对象的边界。')
    end
  elsif export_type == '分别输出'
    output_dir = UI.savepanel('选择并输入保存的文件夹名称', '', '')

    if output_dir.nil?
      return
    end

    saved_count = 0  # 记录成功保存的文件数量

    valid_entities.each_with_index do |entity, index|
      definition = model.definitions.add("Exported_Group_#{index + 1}")
      origin_transformation = case position_option
                             when '保持当前位置'
                               Geom::Transformation.new
                             when '对象坐标原点'
                               new_origin = entity.transformation.origin
                               Geom::Transformation.new(new_origin.vector_to(Geom::Point3d.new))
                             when '对象中心上'
                               center_top = Geom::Point3d.new(entity.bounds.center.x, entity.bounds.center.y, entity.bounds.max.z)
                               Geom::Transformation.new(center_top.vector_to(Geom::Point3d.new))
                             when '对象中心'
                               center = entity.bounds.center
                               Geom::Transformation.new(center.vector_to(Geom::Point3d.new))
                             when '对象中心下'
                               center_bottom = Geom::Point3d.new(entity.bounds.center.x, entity.bounds.center.y, entity.bounds.min.z)
                               Geom::Transformation.new(center_bottom.vector_to(Geom::Point3d.new))
                             else
                               Geom::Transformation.new
                             end

      new_instance = nil
      if entity.is_a?(Sketchup::Group)
        new_instance = definition.entities.add_instance(entity.entities.parent, origin_transformation * entity.transformation)
      elsif entity.is_a?(Sketchup::ComponentInstance)
        new_instance = definition.entities.add_instance(entity.definition, origin_transformation * entity.transformation)
      end
      # 保留外部材质
      new_instance.material = entity.material if new_instance

      # 设置保存路径，并确保文件扩展名为 .skp
      output_path = File.join(output_dir, "#{File.basename(output_dir)}_#{index + 1}.skp")

      begin
        status = definition.save_as(output_path)
        if status
          saved_count += 1  # 成功保存的文件数增加
        else
          UI.messagebox("保存失败：#{output_path}")
        end
      rescue => e
        UI.messagebox("保存失败，出现错误：#{e.message}")
      end

      # 删除临时组件定义
      model.definitions.remove(definition)
    end

    # 如果有成功保存的文件，显示保存目录和保存的文件数
    if saved_count > 0
      saved_files_message = "保存路径：\n#{output_dir}\n\n已成功保存 #{saved_count} 个文件。"
      UI.messagebox(saved_files_message)
    end

  end
end
